/** @jsx jsx */
import React, { useRef, useState } from 'react';
import { jsx } from '@emotion/react';
import { injectIntl } from 'react-intl-next';
import { ACTION, ACTION_SUBJECT, ACTION_SUBJECT_ID, EVENT_TYPE } from '@atlaskit/editor-common/analytics';
import { useSharedPluginState } from '@atlaskit/editor-common/hooks';
import { highlightMessages as messages } from '@atlaskit/editor-common/messages';
import { expandIconWrapperStyle, separatorStyles } from '@atlaskit/editor-common/styles';
import { ColorPalette, getSelectedRowAndColumnFromPalette, highlightColorPalette, REMOVE_HIGHLIGHT_COLOR } from '@atlaskit/editor-common/ui-color';
import { ArrowKeyNavigationType, DropdownContainer as Dropdown, TOOLBAR_BUTTON, ToolbarButton } from '@atlaskit/editor-common/ui-menu';
import { hexToEditorTextBackgroundPaletteColor } from '@atlaskit/editor-palette';
import { akEditorMenuZIndex } from '@atlaskit/editor-shared-styles';
import ExpandIcon from '@atlaskit/icon/glyph/chevron-down';
import { Flex } from '@atlaskit/primitives';
import { changeColor } from '../commands';
import { EditorHighlightIcon } from './EditorHighlightIcon';
const ToolbarHighlightColor = ({
  popupsMountPoint,
  popupsBoundariesElement,
  popupsScrollableElement,
  isToolbarReducedSpacing,
  dispatchAnalyticsEvent,
  disabled,
  pluginInjectionApi,
  intl: {
    formatMessage
  }
}) => {
  const toolbarItemRef = useRef(null);
  const [isDropdownOpen, setIsDropdownOpen] = useState(false);
  const [isOpenedByKeyboard, setIsOpenedByKeyboard] = useState(false);
  const {
    highlightState
  } = useSharedPluginState(pluginInjectionApi, ['highlight']);
  const setDropdownOpen = isOpen => {
    dispatchAnalyticsEvent === null || dispatchAnalyticsEvent === void 0 ? void 0 : dispatchAnalyticsEvent({
      action: isOpen ? ACTION.OPENED : ACTION.CLOSED,
      actionSubject: ACTION_SUBJECT.TOOLBAR,
      actionSubjectId: ACTION_SUBJECT_ID.HIGHLIGHT,
      eventType: EVENT_TYPE.TRACK
    });
    setIsDropdownOpen(isOpen);
  };
  const toggleDropdown = () => setDropdownOpen(!isDropdownOpen);
  const handleClick = () => {
    setIsOpenedByKeyboard(false);
    toggleDropdown();
  };
  const handleKeyDown = event => {
    if (event.key === 'Enter' || event.key === ' ') {
      event.preventDefault();
      setIsOpenedByKeyboard(true);
      toggleDropdown();
    }
  };
  const handleClickOutside = () => {
    if (isDropdownOpen) {
      setDropdownOpen(false);
      setIsOpenedByKeyboard(false);
    }
  };
  const handleEscapeKeydown = () => {
    if (isDropdownOpen) {
      var _toolbarItemRef$curre;
      setDropdownOpen(false);
      setIsOpenedByKeyboard(false);
      toolbarItemRef === null || toolbarItemRef === void 0 ? void 0 : (_toolbarItemRef$curre = toolbarItemRef.current) === null || _toolbarItemRef$curre === void 0 ? void 0 : _toolbarItemRef$curre.focus();
    }
  };
  const handleColorChange = color => {
    var _pluginInjectionApi$c, _pluginInjectionApi$a;
    pluginInjectionApi === null || pluginInjectionApi === void 0 ? void 0 : (_pluginInjectionApi$c = pluginInjectionApi.core) === null || _pluginInjectionApi$c === void 0 ? void 0 : _pluginInjectionApi$c.actions.execute(changeColor(pluginInjectionApi === null || pluginInjectionApi === void 0 ? void 0 : (_pluginInjectionApi$a = pluginInjectionApi.analytics) === null || _pluginInjectionApi$a === void 0 ? void 0 : _pluginInjectionApi$a.actions)({
      color
    }));
    setDropdownOpen(false);
  };

  // Don't render the toolbar option while the plugin is initialising
  if (!highlightState) {
    return null;
  }
  const toolbarButtonLabel = formatMessage(messages.highlight);

  // Get the design token for the  active color (if it exists) to modify the toolbar
  // icon, but show the nice rainbow if none is selected
  const activeColorToken = highlightState.activeColor === REMOVE_HIGHLIGHT_COLOR ? null : hexToEditorTextBackgroundPaletteColor(highlightState.activeColor);

  // pixels, used to determine where to horizontally position the dropdown when space is limited
  // this should reflect the width of the dropdown when fully populated with colors, including translations due to layering
  const fitWidth = 242;
  const {
    selectedRowIndex,
    selectedColumnIndex
  } = getSelectedRowAndColumnFromPalette(highlightColorPalette, highlightState.activeColor);
  return jsx(Flex, {
    alignItems: "center"
  }, jsx(Dropdown, {
    mountTo: popupsMountPoint,
    boundariesElement: popupsBoundariesElement,
    scrollableElement: popupsScrollableElement,
    isOpen: isDropdownOpen && !highlightState.disabled,
    handleClickOutside: handleClickOutside,
    handleEscapeKeydown: handleEscapeKeydown,
    zIndex: akEditorMenuZIndex,
    fitWidth: fitWidth,
    closeOnTab: true,
    arrowKeyNavigationProviderOptions: {
      type: ArrowKeyNavigationType.COLOR,
      selectedRowIndex,
      selectedColumnIndex,
      isOpenedByKeyboard,
      isPopupPositioned: true
    },
    trigger: jsx(ToolbarButton, {
      buttonId: TOOLBAR_BUTTON.HIGHLIGHT,
      spacing: isToolbarReducedSpacing ? 'none' : 'default',
      disabled: disabled || highlightState.disabled,
      selected: isDropdownOpen,
      "aria-label": toolbarButtonLabel,
      "aria-expanded": isDropdownOpen,
      "aria-haspopup": true,
      title: toolbarButtonLabel,
      onClick: handleClick,
      onKeyDown: handleKeyDown,
      ref: toolbarItemRef,
      iconBefore: jsx(Flex, null, jsx(EditorHighlightIcon, {
        selectedColor: activeColorToken,
        disabled: highlightState.disabled
      }), jsx("span", {
        css: expandIconWrapperStyle
      }, jsx(ExpandIcon, {
        label: ""
      })))
    })
  }, jsx("div", {
    "data-testid": "highlight-color-palette"
  }, jsx(ColorPalette, {
    onClick: color => handleColorChange(color),
    selectedColor: highlightState.activeColor,
    paletteOptions: {
      palette: highlightColorPalette,
      hexToPaletteColor: hexToEditorTextBackgroundPaletteColor
    }
  }))), jsx("span", {
    css: separatorStyles
  }));
};
export const ToolbarHighlightColorWithIntl = injectIntl(ToolbarHighlightColor);